/* recfilte.c: tty record filter */
/* $Id: recfilter.c,v 1.8 2003/11/26 07:18:40 knok Exp $ */

/*
Copyright (C) 2003 NOKUBI Takatsugu <knok@daionet.gr.jp>
This is free software with ABSOLUTELY NO WARRANTY.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <termios.h>
#include <signal.h>

#include "udlib.h"

void settermraw(void);
void resetterm(void);
void sighandler(int);

static struct termios tbuf;

#define MAXLINE 2048
char *sockpath = "/tmp/backtick";

int
main(int argc, char *argv[])
{
	unsigned char uc;
	int sfd;

	if (isatty(0)) {
	  settermraw();
	  setbuf(stdout, NULL);
	}
	if (argc > 1)
	  sockpath = argv[1];
	sfd = mkudclid(sockpath);
	if (sfd < 0)
	    return 1;
	while (read(0, &uc, 1) == 1) {
	  if ((uc >= 30 && uc <= 0x7e) ||
	      uc == 0x0a || uc == 0x0d || uc == 0x1b) {
	    char tmp[1024];
	    sprintf(tmp, "%c\n", uc);
	    udsendto(sfd, tmp);
	  }
	  fputc(uc, stdout);
	}

	resetterm();
        return 0;
}

void settermraw()
{
  struct termios term;
  if (tcgetattr(0, &term) < 0)
    return;
  memcpy(&tbuf, &term, sizeof(tbuf));

  signal(SIGINT, sighandler);
  signal(SIGQUIT, sighandler);
  signal(SIGTERM, sighandler);
  signal(SIGHUP, sighandler);

  term.c_lflag &= ~(ICANON | ISIG | ECHO | IEXTEN);
  term.c_iflag &= ~(INPCK | ICRNL | ISTRIP | IXON | BRKINT);
  term.c_oflag &= ~OPOST;
  term.c_cflag &= ~(CSIZE | PARENB);
  term.c_cflag |= CS8;
  term.c_cc[VMIN] = 1;
  term.c_cc[VTIME] = 0;
  tcsetattr(0, TCSAFLUSH, &term);
}

void resetterm()
{
  if (tcgetattr(0, &tbuf) < 0)
    return;
}

void sighandler(int s)
{
  resetterm();
  exit(0);
}
